package com.klab.security.common.exp;

import cn.hutool.json.JSONUtil;
import com.cloud.kicc.common.core.constant.SecurityConstants;
import com.cloud.kicc.common.core.enums.CasSystemEnum;
import com.cloud.kicc.common.core.jackson.KiccJavaTimeModule;
import com.cloud.kicc.common.data.entity.CasUser;
import com.cloud.kicc.common.data.entity.KicsUser;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.klab.security.common.exception.UnConfiguredUserDataException;
import com.klab.security.common.override.jackson2.SimpleGrantedAuthorityMixin;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.beans.BeanUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.exceptions.InvalidTokenException;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2Request;
import org.springframework.security.oauth2.provider.token.ResourceServerTokenServices;
import org.springframework.security.oauth2.provider.token.TokenStore;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 *<p>
 * 本地资源服务器令牌服务
 *</p>
 *
 * @Author: wangxiang4
 * @Since: 2023/8/24
 */
@RequiredArgsConstructor
public class KiccLocalResourceServerTokenServices implements ResourceServerTokenServices {

	private final TokenStore tokenStore;

	private final JdbcTemplate jdbcTemplate;

	@SneakyThrows
	public OAuth2Authentication loadAuthentication(String accessToken) throws AuthenticationException, InvalidTokenException {
		// 根据token加载身份验证
		OAuth2Authentication oAuth2Authentication = tokenStore.readAuthentication(accessToken);
		if (oAuth2Authentication == null) {
			return null;
		}
		OAuth2Request oAuth2Request = oAuth2Authentication.getOAuth2Request();
		// 检测是否是属于认证的CasUser实体用户
		if (!(oAuth2Authentication.getPrincipal() instanceof CasUser)) {
			return oAuth2Authentication;
		}
		CasUser casUser = (CasUser) oAuth2Authentication.getPrincipal();

		// 设置SSO子系统扩展用户信息
		if (casUser.getExPrincipals().get(CasSystemEnum.KICS) == null) {
			List<KicsUser> userList = jdbcTemplate.query(com.klab.security.common.constant.SecurityConstants.SSO_USER_FIND_STATEMENT, new BeanPropertyRowMapper<>(KicsUser.class), casUser.getId());
			if (userList.isEmpty()) throw new UnConfiguredUserDataException("System user not found Contact your system administrator for configuration!");
			KicsUser user = userList.get(0);
			List<String> permissions = jdbcTemplate.queryForList(com.klab.security.common.constant.SecurityConstants.SSO_PERMISSION_FIND_STATEMENT, String.class, user.getUserId());
			List<SimpleGrantedAuthority> authorities = Arrays.stream(permissions.toArray(new String[0]))
					.map(SimpleGrantedAuthority::new)
					.collect(Collectors.toList());
			KicsUser kicsUser = new KicsUser(
					casUser.getUsername(),
					casUser.getPassword(),
					casUser.isEnabled(),
					casUser.isAccountNonExpired(),
					casUser.isCredentialsNonExpired(),
					casUser.isAccountNonLocked(),
					authorities
			);
			kicsUser.setPermissions(permissions.toArray(new String[0]));
			BeanUtils.copyProperties(user, kicsUser);
			casUser.getExPrincipals().put(CasSystemEnum.KICS, new ObjectMapper()
					.registerModule(new KiccJavaTimeModule())
					.setSerializationInclusion(JsonInclude.Include.NON_NULL)
					.writeValueAsString(kicsUser));
			tokenStore.storeAccessToken(tokenStore.getAccessToken(oAuth2Authentication), oAuth2Authentication);
		}

		// 覆盖casUser核心authorities
		String str = casUser.getExPrincipals().get(CasSystemEnum.KICS);
		if (!JSONUtil.isTypeJSON(str)) throw new UnConfiguredUserDataException("exPrincipals not json strings!");
		KicsUser kicsUser = new ObjectMapper()
				.registerModule(new KiccJavaTimeModule())
				.addMixIn(SimpleGrantedAuthority.class, SimpleGrantedAuthorityMixin.class)
				.readValue(str, KicsUser.class);
		CasUser exCasUser = new CasUser(
				casUser.getUsername(),
				SecurityConstants.MOCK_PASSWORD,
				casUser.isEnabled(),
				casUser.isAccountNonExpired(),
				casUser.isCredentialsNonExpired(),
				casUser.isAccountNonLocked(),
				kicsUser.getAuthorities());
		BeanUtils.copyProperties(casUser, exCasUser);

		// 每次请求前都预先加载用户名密码身份验证令牌
		Authentication userAuthentication = new UsernamePasswordAuthenticationToken(exCasUser, "N/A", exCasUser.getAuthorities());
		OAuth2Authentication authentication = new OAuth2Authentication(oAuth2Request, userAuthentication);
		authentication.setAuthenticated(true);
		return authentication;
	}

	@Override
	public OAuth2AccessToken readAccessToken(String accessToken) {
		throw new UnsupportedOperationException("Not supported: read access token");
	}

}
