package com.klab.security.common.util;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.cloud.kicc.common.core.enums.CasSystemEnum;
import com.cloud.kicc.common.core.jackson.KiccJavaTimeModule;
import com.cloud.kicc.common.core.util.SpringContextHolderUtil;
import com.cloud.kicc.common.data.entity.CasUser;
import com.cloud.kicc.common.data.entity.KicsUser;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.klab.security.common.constant.SecurityConstants;
import com.klab.security.common.override.jackson2.SimpleGrantedAuthorityMixin;
import lombok.SneakyThrows;
import lombok.experimental.UtilityClass;
import org.springframework.beans.BeanUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 *<p>
 * 安全工具类
 *</p>
 *
 * @Author: wangxiang4
 * @Since: 2023/8/24
 */
@UtilityClass
public class SecurityUtils {

	/**
	 * 获取Authentication
	 */
	public Authentication getAuthentication() {
		return SecurityContextHolder.getContext().getAuthentication();
	}

	/**
	 * 获取用户
	 */
	public CasUser getCasUser(Authentication authentication) {
		Object principal = authentication.getPrincipal();
		if (principal instanceof CasUser) {
			return (CasUser) principal;
		}
		return null;
	}

	/**
	 * 获取CAS用户
	 */
	public CasUser getCasUser() {
		Authentication authentication = getAuthentication();
		if (authentication == null) {
			return null;
		}
		return getCasUser(authentication);
	}

	/**
	 * 获取完整用户
	 * @param casSystemEnum SSO系统枚举
	 * @param valueType 自定义扩展用户
	 * @return T
	 */
	@SneakyThrows
	public <T> T getUser(CasSystemEnum casSystemEnum, Class<T> valueType) {
		CasUser casUser = getCasUser();
		if (casUser == null) return null;
		String str = casUser.getExPrincipals().get(casSystemEnum);
		if (JSONUtil.isJson(str))
			return new ObjectMapper()
					.registerModule(new KiccJavaTimeModule())
					.addMixIn(SimpleGrantedAuthority.class, SimpleGrantedAuthorityMixin.class)
					.readValue(str, valueType);
		return null;
	}

	/**
	 * 获取SSO扩展用户
	 */
	public KicsUser getUser() {
		return getUser(CasSystemEnum.KICS, KicsUser.class);
	}

	/**
	 * 获取用户角色信息
	 * @return 角色集合
	 */
	public List<String> getRoles() {
		Authentication authentication = getAuthentication();
		Collection<? extends GrantedAuthority> authorities = authentication.getAuthorities();

		List<String> roleIds = new ArrayList<>();
		authorities.stream().filter(granted -> StrUtil.startWith(granted.getAuthority(), SecurityConstants.ROLE))
				.forEach(granted -> {
					String id = StrUtil.removePrefix(granted.getAuthority(), SecurityConstants.ROLE);
					roleIds.add(id);
				});
		return roleIds;
	}

	/**
	 * 对外开放接口临时登录会话
	 * @param userId 用户id
	 * @return User 用户对象
	 */
	@SneakyThrows
	public KicsUser openInterfaceTemporaryLoginSession(String userId) {
		KicsUser user = SpringContextHolderUtil.getBean(JdbcTemplate.class).queryForObject(SecurityConstants.SSO_USER_FIND_STATEMENT, KicsUser.class, userId);
		if (user == null) throw new RuntimeException("用户不存在");
		List<String> permissions = SpringContextHolderUtil.getBean(JdbcTemplate.class).queryForList(SecurityConstants.SSO_PERMISSION_FIND_STATEMENT, String.class, userId);
		List<SimpleGrantedAuthority> authorities = Arrays.stream(permissions.toArray(new String[0]))
				.map(SimpleGrantedAuthority::new)
				.collect(Collectors.toList());

		CasUser casUser = new CasUser(
				user.getUsername(),
				SecurityConstants.BCRYPT + user.getPassword(),
				true,
				true,
				true,
				StrUtil.equals(user.getSsoStatus(), "0"),
				authorities,
				user.getCasUserId(),
				user.getNickName(),
				user.getEmail(),
				user.getPhone(),
				user.getSex(),
				user.getAvatar(),
				user.getLoginIp(),
				user.getLoginTime(),
				user.getSsoStatus(),
				user.getSsoCreateById(),
				user.getSsoCreateByName(),
				user.getSsoCreateTime(),
				user.getSsoUpdateById(),
				user.getSsoUpdateByName(),
				user.getSsoUpdateTime(),
				user.getRemarks()
		);

		KicsUser kicsUser = new KicsUser(
				casUser.getUsername(),
				casUser.getPassword(),
				casUser.isEnabled(),
				casUser.isAccountNonExpired(),
				casUser.isCredentialsNonExpired(),
				casUser.isAccountNonLocked(),
				authorities
		);
		BeanUtils.copyProperties(user, kicsUser);
		// 设置扩展用户数据
		casUser.getExPrincipals().put(CasSystemEnum.KICS, new ObjectMapper()
				.registerModule(new KiccJavaTimeModule())
				.setSerializationInclusion(JsonInclude.Include.NON_NULL)
				.writeValueAsString(kicsUser));
		Authentication authentication = new UsernamePasswordAuthenticationToken(casUser, "N/A", casUser.getAuthorities());
		SecurityContextHolder.getContext().setAuthentication(authentication);
		return user;
	}

	public static void main(String[] args) {
		System.out.println(new BCryptPasswordEncoder().encode("123456"));
	}

}
